/*
 *		This Code Was Created By Steve Wortham
 */


#include <math.h>		    // Header File For Math functions
#include <windows.h>		// Header File For Windows
#include <stdio.h>			// Header File For Standard Input/Output
#include <gl\gl.h>			// Header File For The OpenGL32 Library
#include <gl\glu.h>			// Header File For The GLu32 Library
#include <gl\glaux.h>		// Header File For The Glaux Library

HDC		hDC=NULL;			// Private GDI Device Context
HGLRC	hRC=NULL;			// Permanent Rendering Context
HWND	hWnd=NULL;			// Holds Our Window Handle

bool	keys[256];			// Array Used For The Keyboard Routine
bool	active=TRUE;		// Window Active Flag Set To TRUE By Default
bool	fullscreen=TRUE;	// Fullscreen Flag Set To Fullscreen Mode By Default

GLfloat xtrans = 0;
GLfloat xptrans = 0;
GLfloat ztrans = 0;
GLfloat zptrans = 0;

int loop;
float V;
float Angle;
float AngleZ;

GLUquadricObj *quadratic;	// Storage For Our Quadratic Objects ( NEW )
GLuint	texture[4];

// Absolute value function(comes in handy later).
GLfloat ABS(GLfloat A)
{
  if (A < 0)
  A = -A; 
  return A;
}

#define	MAX_PARTICLES	50

GLfloat MAX=15; 

typedef struct						// Create A Structure For Particle
{
	float	x;						// X Position
	float	y;						// Y Position
	float	z;						// Z Position
	float	xi;						// X Direction
	float	yi;						// Y Direction
	float	zi;						// Z Direction
	float	yg;						// Y Gravity
}
particles;							// Particles Structure

particles particle[MAX_PARTICLES];	// Particle Array (Room For Particle Info)

GLuint	fogMode[]= { GL_EXP, GL_EXP2, GL_LINEAR };	// Storage For Three Types Of Fog
GLuint	fogfilter = 0;								// Which Fog Mode To Use 
GLfloat	fogColor[4] = {0, 0, 0, 0};		            // Fog Color

LRESULT	CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);	// Declaration For WndProc

AUX_RGBImageRec *LoadBMP(char *Filename)				// Loads A Bitmap Image
{
	FILE *File=NULL;									// File Handle

	if (!Filename)										// Make Sure A Filename Was Given
	{
		return NULL;									// If Not Return NULL
	}

	File=fopen(Filename,"r");							// Check To See If The File Exists

	if (File)											// Does The File Exist?
	{
		fclose(File);									// Close The Handle
		return auxDIBImageLoad(Filename);				// Load The Bitmap And Return A Pointer
	}

	return NULL;										// If Load Failed Return NULL
}


int LoadGLTextures()									// Load Bitmap And Convert To A Texture
{
        int Status=FALSE;								// Status Indicator
        AUX_RGBImageRec *TextureImage[1];				// Create Storage Space For The Textures
        memset(TextureImage,0,sizeof(void *)*1);		// Set The Pointer To NULL


        if (TextureImage[0]=LoadBMP("texture/wood.bmp"))	// Load Particle Texture
        {
			Status=TRUE;								// Set The Status To TRUE
			glGenTextures(1, &texture[1]);				// Create One Texture

			// Create MipMapped Texture
     		glBindTexture(GL_TEXTURE_2D, texture[1]);
	    	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	    	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
	    	gluBuild2DMipmaps(GL_TEXTURE_2D, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);           
        }

        if (TextureImage[0])							// If Texture Exists
		{
			if (TextureImage[0]->data)					// If Texture Image Exists
			{
				free(TextureImage[0]->data);			// Free The Texture Image Memory
			}
			free(TextureImage[0]);						// Free The Image Structure
		}


        if (TextureImage[0]=LoadBMP("texture/wood2.bmp"))	// Load Particle Texture
        {
			Status=TRUE;								// Set The Status To TRUE
			glGenTextures(1, &texture[2]);				// Create One Texture

			// Create MipMapped Texture
    		glBindTexture(GL_TEXTURE_2D, texture[2]);
	    	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	    	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
	    	gluBuild2DMipmaps(GL_TEXTURE_2D, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);           
        }

        if (TextureImage[0])							// If Texture Exists
		{
			if (TextureImage[0]->data)					// If Texture Image Exists
			{
				free(TextureImage[0]->data);			// Free The Texture Image Memory
			}
			free(TextureImage[0]);						// Free The Image Structure
		}

        if (TextureImage[0]=LoadBMP("texture/plasma.bmp"))	// Load Particle Texture
        {
			Status=TRUE;								// Set The Status To TRUE
			glGenTextures(1, &texture[3]);				// Create One Texture

			// Create MipMapped Texture
  		    glBindTexture(GL_TEXTURE_2D, texture[3]);
			
            glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
			glTexImage2D(GL_TEXTURE_2D, 0, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);
        }

        if (TextureImage[0])							// If Texture Exists
		{
			if (TextureImage[0]->data)					// If Texture Image Exists
			{
				free(TextureImage[0]->data);			// Free The Texture Image Memory
			}
			free(TextureImage[0]);						// Free The Image Structure
		}

        return Status;									// Return The Status
}


GLvoid ReSizeGLScene(GLsizei width, GLsizei height)		// Resize And Initialize The GL Window
{
	if (height==0)										// Prevent A Divide By Zero By
	{
		height=1;										// Making Height Equal One
	}

	glViewport(0,0,width,height);						// Reset The Current Viewport

	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix

	// Calculate The Aspect Ratio Of The Window
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.1f,200.0f);

	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glLoadIdentity();									// Reset The Modelview Matrix
}


int InitGL(GLvoid)										// All Setup For OpenGL Goes Here
{
	if (!LoadGLTextures())								// Jump To Texture Loading Routine
	{
		return FALSE;									// If Texture Didn't Load Return FALSE
	}
    
	glEnable(GL_TEXTURE_2D);							// Enable Texture Mapping

	glClearColor(0, 0, 0, 1.0f);				// Black Background
	glClearDepth(1);		
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// Really Nice Perspective Calculations
        
	glFogi(GL_FOG_MODE, fogMode[1]);			// Fog Mode
	glFogfv(GL_FOG_COLOR, fogColor);					// Set Fog Color
	glFogf(GL_FOG_DENSITY, 0.025f);						// How Dense Will The Fog Be
	glHint(GL_FOG_HINT, GL_DONT_CARE);					// Fog Hint Value
	glFogf(GL_FOG_START, 0.0f);							// Fog Start Depth
	glFogf(GL_FOG_END, 10.0f);							// Fog End Depth
	glEnable(GL_FOG);									// Enables GL_FOG

    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA,GL_DST_ALPHA);					// Type Of Blending To Perform

	quadratic=gluNewQuadric();							// Create A Pointer To The Quadric Object (Return 0 If No Memory) (NEW)
	gluQuadricNormals(quadratic, GLU_SMOOTH);			// Create Smooth Normals (NEW)
	gluQuadricTexture(quadratic, GL_TRUE);				// Create Texture Coords (NEW)   

    for (loop=0;loop<MAX_PARTICLES;loop++)				// Initializes All The Textures
	{
        particle[loop].x = float((rand()%20)-10);
        particle[loop].z = float((rand()%20)-10);

        V = float((rand()%250))/15;
		Angle = float(rand()%360);
                        
        particle[loop].xi = float(rand()%150)/10+5;
        particle[loop].yi = float(cos(Angle) * V/2);
        particle[loop].zi = float(rand()%150)/10+5;
        
		particle[loop].yg=-0.02f;						// Set Vertical Pull Downward
	}   
   
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   
	return TRUE;										// Initialization Went OK
}


GLfloat	yrot=0;				// Y Rotation

const float piover180 = 0.0174532925f;
GLfloat XP=0;
GLfloat ZP=0;

GLfloat sceneroty;
GLfloat heading;
GLfloat walkbias = 0;
GLfloat walkbiasangle = 0;
GLfloat zprot;

GLfloat life = 1;
GLfloat XPOS = 0;
GLfloat ZPOS = 0;
GLfloat size = .25f;


int DrawGLScene(GLvoid)									// Here's Where We Do All The Drawing
{
    glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);	// Clear The Screen And The Depth Buffer

    XP *= 0.95f; 
	ZP *= 0.95f; 
    xtrans -= XP/10;
    ztrans -= ZP/10;
    
	if(xtrans > 14.5f && XP <= 0) xtrans=14.5f, XP = 0;
    else if(xtrans < -14.5f && XP >= 0) xtrans=-14.5f, XP = 0;
	if(ztrans > 14.5f && ZP <= 0) ztrans=14.5f, ZP = 0;
    else if(ztrans < -14.5f && ZP >= 0) ztrans=-14.5f, ZP = 0;

    zprot*=.9f;
    heading += zprot;
    yrot = heading;
	
	sceneroty = 360.0f - yrot;

    glLoadIdentity();
    glRotatef(sceneroty,0,1.f,0);
    glTranslatef(xtrans,1,ztrans);
    
      glBindTexture(GL_TEXTURE_2D, texture[1]);
      glColor4f(1.f,.5f,.5f,.35f);
      glBegin(GL_TRIANGLE_STRIP);
        glTexCoord2f(3,3);  glVertex3f(15 ,-2,15);
        glTexCoord2f(0,3);  glVertex3f(-15,-2,15);
        glTexCoord2f(3,0);  glVertex3f(15 ,-2,-15);
        glTexCoord2f(0,0);  glVertex3f(-15,-2,-15);
      glEnd();

      glColor4f(1,.75f,.5f,.35f);
      glBegin(GL_TRIANGLE_STRIP);
        glTexCoord2f(3,3);  glVertex3f(15 ,2,15);
        glTexCoord2f(0,3);  glVertex3f(-15,2,15);
        glTexCoord2f(3,0);  glVertex3f(15 ,2,-15);
        glTexCoord2f(0,0);  glVertex3f(-15,2,-15);
      glEnd();

      glBindTexture(GL_TEXTURE_2D, texture[2]);
      glColor4f(.7f,.5f,.45f,.75f);

      glBegin(GL_TRIANGLE_STRIP);
        glTexCoord2f(15,1);  glVertex3f(15 ,2 ,-15);
        glTexCoord2f(0,1);   glVertex3f(-15,2 ,-15);
        glTexCoord2f(15,0);  glVertex3f(15 ,-2,-15);
        glTexCoord2f(0,0);   glVertex3f(-15,-2,-15);
      glEnd();

      glBegin(GL_TRIANGLE_STRIP);
        glTexCoord2f(15,1);  glVertex3f(15 ,2 ,15);
        glTexCoord2f(0,1);   glVertex3f(-15,2 ,15);
        glTexCoord2f(15,0);  glVertex3f(15 ,-2,15);
        glTexCoord2f(0,0);   glVertex3f(-15,-2,15);
      glEnd();

      glBegin(GL_TRIANGLE_STRIP);
        glTexCoord2f(15,1);  glVertex3f(-15,2 ,15);
        glTexCoord2f(0,1);   glVertex3f(-15,2 ,-15);
        glTexCoord2f(15,0);  glVertex3f(-15,-2,15);
        glTexCoord2f(0,0);   glVertex3f(-15,-2,-15);
      glEnd();

      glBegin(GL_TRIANGLE_STRIP);
        glTexCoord2f(15,1);  glVertex3f(15,2 ,15);
        glTexCoord2f(0,1);   glVertex3f(15,2 ,-15);
        glTexCoord2f(15,0);  glVertex3f(15,-2,15);
        glTexCoord2f(0,0);   glVertex3f(15,-2,-15);
      glEnd();
      
    glBindTexture(GL_TEXTURE_2D, texture[3]);
    GLfloat xdist,ydist,zdist;

   	for (loop=0;loop<MAX;loop++)					// Loop Through All The Particles
	{
       		
            glLoadIdentity();
            glRotatef(sceneroty,0,1,0);
            glTranslatef(xtrans,1,ztrans);            

			float X=particle[loop].x;						// Grab Our Particle X Position
			float Y=particle[loop].y;						// Grab Our Particle Y Position
			float Z=particle[loop].z;					// Particle Z Pos + Zoom
//GRAVITY between lights
// /*
            for (int loop2=0;loop2<MAX;loop2++) 
            {
              if (loop2 != loop)
              { 
                xdist = particle[loop2].x-particle[loop].x;
                ydist = particle[loop2].y-particle[loop].y;
                zdist = particle[loop2].z-particle[loop].z;
                V = 1.0f/(hypot(hypot(xdist,ydist),zdist)+0.1f);
                particle[loop].xi+=xdist*V/50.0f;
                particle[loop].yi+=ydist*V/50.0f;
                particle[loop].zi+=zdist*V/50.0f;
              }  

            }
// */            

            float r,g,b;
            if (loop%3==0)
            {
              r = 1;
              g =.5;
              b = 0; 
            } 
            else if (loop%3==1)
            {
              r = 1;
              g = 0;
              b = 0; 
            } 
            else if (loop%3==2)
            {
              r = 1;
              g = 1;
              b = 0; 
            }                        

//////////////////////////////////////////////////////////////////////////////                      
// HERE IT IS!  Simple isn't it?  The light maps are produced with a simple 
// blurred circular texture mapped onto a square(I use Triangle strips for performance).
// Also, to improve the effect, the lightmap brightens as the light gets closer to the
// floor or ceiling.  That is why I use "life*(ABS((Y-2))/4)".
// Note:  You must have GL_BLEND enabled to perform this effect.
//////////////////////////////////////////////////////////////////////////////

            // FLOOR
            glColor4f(r,g,b,life*(ABS((Y-2))/4));
            glBegin(GL_TRIANGLE_STRIP);
              glTexCoord2f(1,1);  glVertex3f(X+2,-2,Z+2);
              glTexCoord2f(0,1);  glVertex3f(X-2,-2,Z+2);
              glTexCoord2f(1,0);  glVertex3f(X+2,-2,Z-2);
              glTexCoord2f(0,0);  glVertex3f(X-2,-2,Z-2);
            glEnd();

            // CEILING
            glColor4f(r,g,b,life*(ABS((Y+2))/4));
            glBegin(GL_TRIANGLE_STRIP);
              glTexCoord2f(1,1);  glVertex3f(X+2,2,Z+2);
              glTexCoord2f(0,1);  glVertex3f(X-2,2,Z+2);
              glTexCoord2f(1,0);  glVertex3f(X+2,2,Z-2);
              glTexCoord2f(0,0);  glVertex3f(X-2,2,Z-2);
            glEnd();
////////////////////////////////////////////////////////////////////////////
// End of LightMap
////////////////////////////////////////////////////////////////////////////

            glTranslatef(X,Y,Z);
            glColor4f(r,g,b,1.f);        
            gluSphere(quadratic,size,16,16);           
         
            particle[loop].x+=particle[loop].xi/250;// Move On The X Axis By X Speed
			particle[loop].y+=particle[loop].yi/250;// Move On The Y Axis By Y Speed
			particle[loop].z+=particle[loop].zi/250;// Move On The Z Axis By Z Speed           
            //GRAVITY
            particle[loop].yi+=particle[loop].yg;

            if (particle[loop].y > 2-size)
            {
              particle[loop].y=2-size;
              particle[loop].yi=-particle[loop].yi;
            }

            else if (particle[loop].y < -2+size)
            {
              particle[loop].y=-2+size;
              particle[loop].yi=-particle[loop].yi;
            }

            if (particle[loop].x > 15-size)
            {
              particle[loop].x = 15-size;
              particle[loop].xi = -particle[loop].xi;
            }
            else if (particle[loop].x < -15+size)
            {
              particle[loop].x = -15+size;
              particle[loop].xi = -particle[loop].xi;
            }
            if (particle[loop].z > 15-size)
            {
              particle[loop].z = 15-size;
              particle[loop].zi = -particle[loop].zi;
            }
            else if (particle[loop].z < -15+size)
            {
              particle[loop].z = -15+size;
              particle[loop].zi = -particle[loop].zi;
            }
                   
    }

   
   return TRUE;										// Keep Going
}

GLvoid KillGLWindow(GLvoid)								// Properly Kill The Window
{
	if (hRC)											// Do We Have A Rendering Context?
	{
		if (!wglMakeCurrent(NULL,NULL))					// Are We Able To Release The DC And RC Contexts?
		{
			MessageBox(NULL,"Release Of DC And RC Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		}

		if (!wglDeleteContext(hRC))						// Are We Able To Delete The RC?
		{
			MessageBox(NULL,"Release Rendering Context Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		}
		hRC=NULL;										// Set RC To NULL
	}

	if (hDC && !ReleaseDC(hWnd,hDC))					// Are We Able To Release The DC
	{
		MessageBox(NULL,"Release Device Context Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hDC=NULL;										// Set DC To NULL
	}

	if (hWnd && !DestroyWindow(hWnd))					// Are We Able To Destroy The Window?
	{
		MessageBox(NULL,"Could Not Release hWnd.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hWnd=NULL;										// Set hWnd To NULL
	}

	if (fullscreen)										// Are We In Fullscreen Mode?
	{
		ChangeDisplaySettings(NULL,0);					// If So Switch Back To The Desktop
		ShowCursor(TRUE);								// Show Mouse Pointer
	}
}

/*	This Code Creates Our OpenGL Window.  Parameters Are:					*
 *	title			- Title To Appear At The Top Of The Window				*
 *	width			- Width Of The GL Window Or Fullscreen Mode				*
 *	height			- Height Of The GL Window Or Fullscreen Mode			*
 *	bits			- Number Of Bits To Use For Color (8/16/24/32)			*
 *	fullscreenflag	- Use Fullscreen Mode (TRUE) Or Windowed Mode (FALSE)	*/
 
BOOL CreateGLWindow(char* title, int width, int height, int bits, bool fullscreenflag)
{
	GLuint		PixelFormat;			// Holds The Results After Searching For A Match
	HINSTANCE	hInstance;				// Holds The Instance Of The Application
	WNDCLASS	wc;						// Windows Class Structure
	DWORD		dwExStyle;				// Window Extended Style
	DWORD		dwStyle;				// Window Style

	fullscreen=fullscreenflag;			// Set The Global Fullscreen Flag

	hInstance			= GetModuleHandle(NULL);				// Grab An Instance For Our Window
	wc.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;	// Redraw On Size, And Own DC For Window.
	wc.lpfnWndProc		= (WNDPROC) WndProc;					// WndProc Handles Messages
	wc.cbClsExtra		= 0;									// No Extra Window Data
	wc.cbWndExtra		= 0;									// No Extra Window Data
	wc.hInstance		= hInstance;							// Set The Instance
	wc.hIcon			= LoadIcon(NULL, IDI_WINLOGO);			// Load The Default Icon
	wc.hCursor			= LoadCursor(NULL, IDC_ARROW);			// Load The Arrow Pointer
	wc.hbrBackground	= NULL;									// No Background Required For GL
	wc.lpszMenuName		= NULL;									// We Don't Want A Menu
	wc.lpszClassName	= "OpenGL";								// Set The Class Name

	if (!RegisterClass(&wc))									// Attempt To Register The Window Class
	{
		MessageBox(NULL,"Failed To Register The Window Class.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;											// Return FALSE
	}
	
	if (fullscreen)												// Attempt Fullscreen Mode?
	{
		DEVMODE dmScreenSettings;								// Device Mode
		memset(&dmScreenSettings,0,sizeof(dmScreenSettings));	// Makes Sure Memory's Cleared
		dmScreenSettings.dmSize=sizeof(dmScreenSettings);		// Size Of The Devmode Structure
		dmScreenSettings.dmPelsWidth	= width;				// Selected Screen Width
		dmScreenSettings.dmPelsHeight	= height;				// Selected Screen Height
		dmScreenSettings.dmBitsPerPel	= bits;					// Selected Bits Per Pixel
		dmScreenSettings.dmFields=DM_BITSPERPEL|DM_PELSWIDTH|DM_PELSHEIGHT;

		// Try To Set Selected Mode And Get Results.  NOTE: CDS_FULLSCREEN Gets Rid Of Start Bar.
		if (ChangeDisplaySettings(&dmScreenSettings,CDS_FULLSCREEN)!=DISP_CHANGE_SUCCESSFUL)
		{
			// If The Mode Fails, Offer Two Options.  Quit Or Use Windowed Mode.
			if (MessageBox(NULL,"The Requested Fullscreen Mode Is Not Supported By\nYour Video Card. Use Windowed Mode Instead?","NeHe GL",MB_YESNO|MB_ICONEXCLAMATION)==IDYES)
			{
				fullscreen=FALSE;		// Windowed Mode Selected.  Fullscreen = FALSE
			}
			else
			{
				// Pop Up A Message Box Letting User Know The Program Is Closing.
				MessageBox(NULL,"Program Will Now Close.","ERROR",MB_OK|MB_ICONSTOP);
				return FALSE;									// Return FALSE
			}
		}
	}

	if (fullscreen)												// Are We Still In Fullscreen Mode?
	{
		dwExStyle=WS_EX_APPWINDOW;								// Window Extended Style
		dwStyle=WS_POPUP | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;	// Windows Style
		ShowCursor(FALSE);										// Hide Mouse Pointer
	}
	else
	{
		dwExStyle=WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;						// Window Extended Style
		dwStyle=WS_OVERLAPPEDWINDOW | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;	// Windows Style
	}

	// Create The Window
	if (!(hWnd=CreateWindowEx(	dwExStyle,			// Extended Style For The Window
								"OpenGL",			// Class Name
								title,				// Window Title
								dwStyle,			// Window Style
								0, 0,				// Window Position
								width, height,		// Selected Width And Height
								NULL,				// No Parent Window
								NULL,				// No Menu
								hInstance,			// Instance
								NULL)))				// Dont Pass Anything To WM_CREATE
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Window Creation Error.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	static	PIXELFORMATDESCRIPTOR pfd=				// pfd Tells Windows How We Want Things To Be
	{
		sizeof(PIXELFORMATDESCRIPTOR),				// Size Of This Pixel Format Descriptor
		1,											// Version Number
		PFD_DRAW_TO_WINDOW |						// Format Must Support Window
		PFD_SUPPORT_OPENGL |						// Format Must Support OpenGL
		PFD_DOUBLEBUFFER,							// Must Support Double Buffering
		PFD_TYPE_RGBA,								// Request An RGBA Format
		bits,										// Select Our Color Depth
		0, 0, 0, 0, 0, 0,							// Color Bits Ignored
		0,											// No Alpha Buffer
		0,											// Shift Bit Ignored
		0,											// No Accumulation Buffer
		0, 0, 0, 0,									// Accumulation Bits Ignored
		16,											// 16Bit Z-Buffer (Depth Buffer)  
		0,											// No Stencil Buffer
		0,											// No Auxiliary Buffer
		PFD_MAIN_PLANE,								// Main Drawing Layer
		0,											// Reserved
		0, 0, 0										// Layer Masks Ignored
	};
	
	if (!(hDC=GetDC(hWnd)))							// Did We Get A Device Context?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Create A GL Device Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if (!(PixelFormat=ChoosePixelFormat(hDC,&pfd)))	// Did Windows Find A Matching Pixel Format?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Find A Suitable PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if(!SetPixelFormat(hDC,PixelFormat,&pfd))		// Are We Able To Set The Pixel Format?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Set The PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if (!(hRC=wglCreateContext(hDC)))				// Are We Able To Get A Rendering Context?
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Create A GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if(!wglMakeCurrent(hDC,hRC))					// Try To Activate The Rendering Context
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Activate The GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	ShowWindow(hWnd,SW_SHOW);						// Show The Window
	SetForegroundWindow(hWnd);						// Slightly Higher Priority
	SetFocus(hWnd);									// Sets Keyboard Focus To The Window
	ReSizeGLScene(width, height);					// Set Up Our Perspective GL Screen

	if (!InitGL())									// Initialize Our Newly Created GL Window
	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Initialization Failed.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	return TRUE;									// Success
}

LRESULT CALLBACK WndProc(	HWND	hWnd,			// Handle For This Window
							UINT	uMsg,			// Message For This Window
							WPARAM	wParam,			// Additional Message Information
							LPARAM	lParam)			// Additional Message Information
{
	switch (uMsg)									// Check For Windows Messages
	{
		case WM_ACTIVATE:							// Watch For Window Activate Message
		{
			if (!HIWORD(wParam))					// Check Minimization State
			{
				active=TRUE;						// Program Is Active
			}
			else
			{
				active=FALSE;						// Program Is No Longer Active
			}

			return 0;								// Return To The Message Loop
		}

		case WM_SYSCOMMAND:							// Intercept System Commands
		{
			switch (wParam)							// Check System Calls
			{
				case SC_SCREENSAVE:					// Screensaver Trying To Start?
				case SC_MONITORPOWER:				// Monitor Trying To Enter Powersave?
				return 0;							// Prevent From Happening
			}
			break;									// Exit
		}

		case WM_CLOSE:								// Did We Receive A Close Message?
		{
			PostQuitMessage(0);						// Send A Quit Message
			return 0;								// Jump Back
		}

		case WM_KEYDOWN:							// Is A Key Being Held Down?
		{
			keys[wParam] = TRUE;					// If So, Mark It As TRUE
			return 0;								// Jump Back
		}

		case WM_KEYUP:								// Has A Key Been Released?
		{
			keys[wParam] = FALSE;					// If So, Mark It As FALSE
			return 0;								// Jump Back
		}

		case WM_SIZE:								// Resize The OpenGL Window
		{
			ReSizeGLScene(LOWORD(lParam),HIWORD(lParam));  // LoWord=Width, HiWord=Height
			return 0;								// Jump Back
		}
	}

	// Pass All Unhandled Messages To DefWindowProc
	return DefWindowProc(hWnd,uMsg,wParam,lParam);
}




int WINAPI WinMain(	HINSTANCE	hInstance,			// Instance
					HINSTANCE	hPrevInstance,		// Previous Instance
					LPSTR		lpCmdLine,			// Command Line Parameters
					int			nCmdShow)			// Window Show State
{
	
    MSG		msg;									// Windows Message Structure
	BOOL	done=FALSE;								// Bool Variable To Exit Loop

    // Ask The User Which Screen Mode They Prefer
    
    fullscreen=true;

    // Ask The User Which Screen Mode They Prefer
	if (MessageBox(NULL,"Would You Like To Run In Fullscreen Mode?", "Start FullScreen?",MB_YESNO|MB_ICONQUESTION)==IDNO)
	{
		fullscreen=FALSE;							// Windowed Mode
	}
	
    
    // Create Our OpenGL Window
	if (!CreateGLWindow("Bouncing Lights",800,600,16,fullscreen))
	{
		return 0;									// Quit If Window Was Not Created
	}

	while(!done)									// Loop That Runs While done=FALSE
	{
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))	// Is There A Message Waiting?
		{
			if (msg.message==WM_QUIT)				// Have We Received A Quit Message?
			{
				done=TRUE;							// If So done=TRUE
			}
			else									// If Not, Deal With Window Messages
			{
				TranslateMessage(&msg);				// Translate The Message
				DispatchMessage(&msg);				// Dispatch The Message
			}
		}
		else										// If There Are No Messages
		{
			
            // Draw The Scene.  Watch For ESC Key And Quit Messages From DrawGLScene()
			if ((active && !DrawGLScene()) || keys[VK_ESCAPE])	// Active?  Was There A Quit Received?
			{
				done=TRUE;							// ESC or DrawGLScene Signalled A Quit
			}
			else									// Not Time To Quit, Update Screen
			{
				SwapBuffers(hDC);					// Swap Buffers (Double Buffering)
				
                if (keys[VK_UP])  // Move forwards
                {
					XP -= (float)sin(heading*piover180) * 0.1f;	
					ZP -= (float)cos(heading*piover180) * 0.1f;
                }
                else if (keys[VK_DOWN]) // Move backwards
                {
					XP += (float)sin(heading*piover180) * 0.1f;	
					ZP += (float)cos(heading*piover180) * 0.1f;
                }	
                if (keys[VK_LEFT]) // Turn left
                {
                  zprot += .15f;               
                }
                else if (keys[VK_RIGHT]) // Turn right
                {
                  zprot -= .15f;
                }                                
                if (keys[VK_ADD])
                {                 
                  MAX+=.1f;                 
                  if (MAX > MAX_PARTICLES) 
                    MAX = MAX_PARTICLES;
                }
                if (keys[VK_SUBTRACT])
                {                 
                  MAX-=.1f;
                  if (MAX < 1) 
                    MAX = 1;                 
                }
                              
			}
		}
	}

	// Shutdown
	KillGLWindow();									// Kill The Window
    return (msg.wParam);							// Exit The Program
}
